/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  bootp.h
 * Purpose: Definitions for implementing the BOOTP protocol
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995-2003 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: bootp.h,v 1.4 2003/01/25 23:29:41 gkminix Exp $
 */

#ifndef _KERNEL_ARPA_BOOTP_H
#define _KERNEL_ARPA_BOOTP_H
#ifndef _USE_ASSEMBLER


/*
 **************************************************************************
 *
 * We need some includes before we can include this file
 */
#include <general.h>



/*
 **************************************************************************
 *
 * BOOTP protocol opcodes:
 */
#define	BOOTP_OP_REQUEST	1	/* BOOTP request		*/
#define	BOOTP_OP_REPLY		2	/* BOOTP reply			*/



/*
 **************************************************************************
 *
 * BOOTP UDP port numbers:
 */
#define BOOTP_S_PORT		67	/* BOOTP server port		*/
#define BOOTP_C_PORT		68	/* BOOTP client port		*/



/*
 **************************************************************************
 *
 * Various definitions:
 */
#define BOOTP_TIMEOUT		4	/* Min # of sec read timeout	*/
#define BOOTP_RETRIES		8	/* No. of retries		*/



/*
 **************************************************************************
 *
 * Vendor magic ID:
 */
#define VM_SIZE			4		/* Size of vendor magic	*/
#define VM_RFC1048		0x63538263L	/* RFC 1048 vendor ID	*/



/*
 **************************************************************************
 *
 * Public BOOTP/DHCP routines local to the ARPA library
 */

/* Retrieve BOOTP vendor extensions file */
extern void get_ext __P((unsigned char *ftag));

/* Set new BOOTP buffer pointer */
extern int new_bootp_buf __P((int id));

/* Set size of new BOOTP buffer */
extern void set_bootp_size __P((void));


#endif				/* _USE_ASSEMBLER */
#endif				/* _KERNEL_ARPA_BOOTP_H */

