/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  arp.h
 * Purpose: Definitions for implementing the ARP protocol
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995-2003 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: arp.h,v 1.4 2003/01/25 23:29:41 gkminix Exp $
 */

#ifndef _KERNEL_NET_ARP_H
#define _KERNEL_NET_ARP_H
#ifndef _USE_ASSEMBLER


/*
 **************************************************************************
 *
 * We need some includes before we can include this file
 */
#include <general.h>
#include <hw/eth.h>



/*
 **************************************************************************
 *
 * ARP timeout in timer ticks when sending a request
 */
#define ARP_TIMEOUT	1		/* Timeout is 1 sec initially	*/
#define ARP_COUNTS	8		/* Max. 8 retries		*/



/*
 **************************************************************************
 *
 * ARP protocol opcodes:
 */
#define	ARPOP_REQUEST	1		/* ARP request			*/
#define	ARPOP_REPLY	2		/* ARP reply			*/



/*
 **************************************************************************
 *
 * ARP protocol address identifiers:
 */
#define ARPPROT_IP	0x0800		/* IP identifier		*/
#define ARPPROT_ARP	0x0806		/* ARP identifier		*/
#define ARPPROT_RARP	0x8035		/* RARP identifier		*/



/*
 **************************************************************************
 *
 * Ethernet ARP header:
 */
struct arphdr
{
	unsigned short	ar_hrd;		/* format of hardware address	*/
	unsigned short	ar_pro;		/* format of protocol address	*/
	unsigned char	ar_hln;		/* length of hardware address	*/
	unsigned char	ar_pln;		/* length of protocol address	*/
	unsigned short	ar_op;		/* ARP opcode (command)		*/
	unsigned char	ar_sha[ETH_ALEN];/* sender hardware address	*/
	unsigned long	ar_sip;          /* sender IP address		*/
	unsigned char	ar_tha[ETH_ALEN];/* target hardware address	*/
	unsigned long	ar_tip;          /* target IP address		*/
};



/*
 **************************************************************************
 * 
 * ARP cache structure
 */
struct arp_cache {
	t_ipaddr ipaddr;			/* IP address		*/
	unsigned char hwaddr[ETH_ALEN];		/* hardware address	*/
};

#define ARPNUM 4		/* number of entries in ARP cache	*/



/*
 **************************************************************************
 *
 * Functions in ARP module:
 */

/* Resolve IP address and return pointer to hardware address */
extern int ip_resolve __P((t_ipaddr myip, t_ipaddr ip, t_ipaddr gw,
						unsigned char **hwaddr));

/* Process a received ARP packet */
extern void ip_recv __P((unsigned int bufsize));


#endif				/* _USE_ASSEMBLER */
#endif				/* _KERNEL_NET_ARP_H */

