/*
 **************************************************************************
 *
 * Boot-ROM-Code to load an operating system across a TCP/IP network.
 *
 * Module:  netpriv.h
 * Purpose: Miscellaeous definitions for the network library
 * Entries: none
 *
 **************************************************************************
 *
 * Copyright (C) 1995-2003 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: netpriv.h,v 1.5 2003/01/25 23:29:41 gkminix Exp $
 */

#ifndef _KERNEL_NET_NETPRIV_H
#define _KERNEL_NET_NETPRIV_H
#ifndef _USE_ASSEMBLER


/*
 **************************************************************************
 * 
 * We need some includes for this file
 */
#include <general.h>
#include <hw/eth.h>



/*
 **************************************************************************
 * 
 * Layout of receive buffer
 */
#define RECVBUF_SIZE	ETH_DATA_MAX

struct iobuf {
	unsigned int  protocol;
	unsigned char buf[RECVBUF_SIZE];
};

extern struct iobuf readbuf;		/* global read buffer		*/



/*
 **************************************************************************
 * 
 * Global variables in the network library
 * Note that these are all in network order!!!
 */
extern unsigned char myhwaddr[ETH_ALEN]; /* my own hardware address	*/
extern unsigned int  mymtu;              /* my own MTU			*/
extern unsigned int  mytype;		 /* my type of network		*/
extern t_ipaddr      myipaddr;           /* my own IP address		*/
extern t_ipaddr      mynetmask;		 /* netmask for my network	*/
extern t_ipaddr      mygateway;		 /* gateway for my network	*/



/*
 **************************************************************************
 *
 * Public routines of the UNDI module:
 */

/* Open UNDI */
extern int undi_open __P((void));

/* Close UNDI */
extern int undi_close __P((void));

/* Send packet using UNDI */
extern int undi_send __P((unsigned char *dest, int proto, int num,
				unsigned int sizes[], unsigned char bufs[]));

/* Receive packet using UNDI */
extern unsigned int undi_recv __P((unsigned int size, unsigned char *buf));



/*
 **************************************************************************
 *
 * External routines in the utility modules:
 */

/* Compute IP header checksum */
extern unsigned int ipchksum __P((unsigned char *buf, unsigned int len));


/* Compute UDP header checksum */
extern unsigned int udpchksum __P((unsigned char *pbuf, unsigned int plen,
				unsigned char *ubuf, unsigned int ulen,
				unsigned char *dbuf, unsigned int dseg,
							unsigned int dlen));


#endif				/* _USE_ASSEMBLER */
#endif				/* _KERNEL_NET_NETPRIV_H */

