!**************************************************************************
!*
!* Boot-ROM-Code to load an operating system across a TCP/IP network.
!*
!* Module:  undi.S
!* Purpose: Interface to the universal network driver interface
!* Entries: _undi_open, _undi_close, _undi_send, _undi_recv,
!*          net_start, net_stop
!*
!**************************************************************************
!*
!* Copyright (C) 1998-2003 Gero Kuhlmann <gero@gkminix.han.de>
!*
!*  This program is free software; you can redistribute it and/or modify
!*  it under the terms of the GNU General Public License as published by
!*  the Free Software Foundation; either version 2 of the License, or
!*  any later version.
!*
!*  This program is distributed in the hope that it will be useful,
!*  but WITHOUT ANY WARRANTY; without even the implied warranty of
!*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*  GNU General Public License for more details.
!*
!*  You should have received a copy of the GNU General Public License
!*  along with this program; if not, write to the Free Software
!*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!*
!* $Id: undi.S,v 1.6 2003/01/25 23:29:41 gkminix Exp $
!*


!
!**************************************************************************
!
! Include assembler header files:
!
#include <general.inc>
#include <memory.inc>
#include <pxe/common.inc>
#include <pxe/undi.inc>
#include <hw/eth.inc>
#include <hw/i8259a.inc>


!
!**************************************************************************
!
! Size of interrupt stack:
!
STACK_SIZE	equ	1024


!
!**************************************************************************
!
! BSS segment.
!
	.bss

	extrn	_myhwaddr		! buffer holding current hardware addr
	extrn	_mymtu			! MTU supported by UNDI
	extrn	_mytype			! type of network interface

	.lcomm	irqnum,2		! IRQ numbers for EOI
	.lcomm	irqmask,2		! IRQ masks
	.lcomm	irqvect,2		! offset to hardware IRQ vector
	.lcomm	oldhwint,4		! old hardware interrupt vector
	.lcomm	intstack,STACK_SIZE	! interrupt stack
	.lcomm	oldsp,4			! old stack pointer
	.lcomm	intcnt,2		! count interrupt requests
	.lcomm	oldmask,1		! old interrupt mask


!
!**************************************************************************
!
! Start code segment.
!
	.text

	public	net_start
	public	net_stop
	public	_undi_open		! define entry points
	public	_undi_close
	public	_undi_send
	public	_undi_recv

! External functions and data within text segment

	extrn	datseg
	extrn	callpxe


!
!**************************************************************************
!
! Startup the network driver interface.
! Input:  none
! Output: AX     -  error code
! Registers changed: (E)AX, (E)BX, CX, SI, DI
!
net_start:

	push	bp

! Initialize the UNDI driver interface.

	sub	sp,#initialize_size
	mov	bp,sp
#ifdef IS386
	mov	dword ptr [bp + o_ui_protocolini],#0
#else
	mov	word ptr [bp + o_ui_protocolini + 0],#0
	mov	word ptr [bp + o_ui_protocolini + 2],#0
#endif
	mov	ax,#UNDI_INITIALIZE
	call	callpxe			! initialize UNDI interface
	add	sp,#initialize_size
	or	ax,ax			! check for error return
	jnz	start9

! Get information about the hardware which is serviced by the UNDI driver,
! like the current hardware address and the hardware interrupt vector.

	sub	sp,#getinfo_size
	mov	bp,sp
	mov	ax,#UNDI_GET_INFORMATION
	call	callpxe			! get UNDI driver information
	or	ax,ax			! check for error return
	jnz	start8

	mov	ax,[bp + o_ugi_mtu]
	cmp	ax,#ETH_DATA_MAX	! check that MTU isnt too large
	ja	start7
	cmp	ax,#ETH_DATA_MIN
	jb	start7
	mov	_mymtu,ax		! save MTU for later

	mov	ax,[bp + o_ugi_hwtype]
	mov	_mytype,ax		! save hardware interface type

	mov	cx,[bp + o_ugi_hwaddrlen]
	cmp	cx,#ETH_ALEN		! check that the hardware address
	jne	start7			! length is correct

#ifdef IS386
if ETH_ALEN = 6
start0	equ	0
else
start0	equ	1
endif
#else
start0	equ	1
#endif

if start0 = 0
	mov	ebx,dword ptr [bp + o_ugi_currentaddr + 0]
	mov	dword ptr [_myhwaddr + 0],ebx
	mov	bx,word ptr [bp + o_ugi_currentaddr + 4]
	mov	word ptr [_myhwaddr + 4],bx
else
	cld
	push	ds
	mov	ax,ss
	mov	ds,ax
	lea	si,[bp + o_ugi_currentaddr]
	mov	di,#_myhwaddr		! copy current hardware address
	rep
	movsb
	pop	ds
endif

! Determine the hardware interrupt number and the address of the interrupt
! vector. We dont have to worry about interrupt 2 on AT and higher systems
! here. The UNDI driver will never return interrupt number 2 on those
! systems, and even if it does the BIOS will redirect interrupt 9 correctly
! and will take care of sending the EOI to the slave.

	mov	ax,[bp + o_ugi_intnumber]
	mov	si,ax
	mov	cx,ax
	mov	ah,#$FF
	cmp	al,#PIC_IRQ_NUM		! set interrupt number for master PIC
	jb	start1
	mov	ah,al			! set interrupt numbers for master and
	sub	ah,#PIC_IRQ_NUM		! slave PIC
	mov	al,#SLAVE_IRQ_NUM
	add	si,#MIN_SLAVE_HWINT - PIC_IRQ_NUM
	jmp	start2
start1:	add	si,#MIN_MASTER_HWINT
start2:	mov	[irqnum],ax		! save IRQ numbers
	mov	ax,#$0001		! compute interrupt mask - this assumes
	shl	ax,cl			! that UNDI never returns irq 2 on AT
	mov	[irqmask],ax		! systems
#ifdef IS186
	shl	si,#2
#else
	shl	si,#1
	shl	si,#1
#endif
	mov	[irqvect],si
	add	sp,#getinfo_size	! cleanup the stack

! Finally redirect the hardware interrupt vector

	call	setint
	xor	ax,ax			! clear carry flag
	jmp	start9

start7:	mov	ax,#PXENV_STATUS_UNDI_CONFIG_DATA
start8:	add	sp,#getinfo_size
start9:	pop	bp
	ret


!
!**************************************************************************
!
! Stop the network driver interface and restore the hardware interrupt
! vector.
! Input:  none
! Output: AX     -  error code
! Registers changed: (E)AX, DX, SI
!
net_stop:

	push	bp

! First restore the hardware interrupt vector. If we cant reset it we
! also cant terminate.

	call	rstint
	or	bx,bx
	mov	ax,#PXENV_STATUS_UNDI_RESOURCES
	jnz	stop9

! Now shutdown the UNDI driver.

	sub	sp,#shutdown_size
	mov	bp,sp
	mov	ax,#UNDI_SHUTDOWN
	call	callpxe			! shutdown UNDI driver
	add	sp,#shutdown_size
	cmp	ax,#PXENV_STATUS_UNDI_NOT_INIT
	jne	stop9
	xor	ax,ax			! dont accept this as an error
stop9:	pop	bp
	ret


!
!**************************************************************************
!
! Open a connection to the UNDI driver.
! Input:  none
! Output: error code (0 if no error)
!
_undi_open:

! Open the interface. Since we dont use multicast addresses, we
! can leave the multicast list empty.

	push	bp
	sub	sp,#open_size
	mov	bp,sp
	mov	word ptr [bp + o_uo_openflag],#0
	mov	word ptr [bp + o_uo_pktfilter],#UNDI_FLTR_DIRECTED + UNDI_FLTR_BROADCAST
	mov	word ptr [bp + o_uo_mcastbuf + o_uma_count],#0
	mov	ax,#UNDI_OPEN
	call	callpxe
	add	sp,#open_size
	pop	bp
	ret


!
!**************************************************************************
!
! Close connection to the UNDI driver.
! Input:  none
! Output: error code (0 if no error)
!
_undi_close:

	push	bp
	sub	sp,#close_size
	mov	bp,sp
	mov	ax,#UNDI_CLOSE
	call	callpxe
	add	sp,#close_size
	pop	bp
	ret


!
!**************************************************************************
!
! Send a packet
! Input:  1. arg  -  near pointer to destination address (0 if broadcast)
!         2. arg  -  protocol of outgoing packet
!         3. arg  -  number of send buffers
!         4. arg  -  near pointer to list of send buffer sizes
!         5. arg  -  near pointer to list of send buffer pointers
! Output: error code
!
_undi_send:

	penter	(transmit_size + tbd_size)	! setup standard stack frame

txbuf	equ	transmit_size
tbd	equ	transmit_size + tbd_size

! Check if we have to send anything at all

	xor	ax,ax
	getcarg	(cx,2)			! get number of send buffers
	jcxz	send9

! Setup the transmit protocol structure

	lea	bx,[bp - tbd]
	getcarg	(dx,1)			! get packet protocol
	getcarg	(ax,0)			! get pointer to destination address
	mov	dh,#UNDI_XMT_BROADCAST
	or	ax,ax			! is it a broadcast address?
	jz	send1
	mov	dh,#UNDI_XMT_DESTADDR
	mov	word ptr [bp - txbuf + o_ut_dest_off],ax
	mov	word ptr [bp - txbuf + o_ut_dest_seg],ds
send1:	mov	word ptr [bp - txbuf + o_ut_protocol],dx
	mov	word ptr [bp - txbuf + o_ut_tbd_off],bx
	mov	word ptr [bp - txbuf + o_ut_tbd_seg],ss

! Setup the transmit data block structure

	cld
	push	si
	push	di
	getcarg	(bx,3)			! get pointer to size list
	getcarg	(si,4)			! get pointer to buffer pointer list
	mov	di,#o_tbd_datablock
	mov	word ptr [bp - tbd + o_tbd_xmtlength],#0
	mov	word ptr [bp - tbd + o_tbd_blocks],cx
send2:	mov	dx,word ptr [bx]
	add	bx,#2
	mov	byte ptr [bp + di - tbd + o_tbd_tdptrtype],#1
	mov	word ptr [bp + di - tbd + o_tbd_tdlength],dx
#ifdef IS386
	lodsd
	mov	dword ptr [bp + di - tbd + o_tbd_tdaddr],eax
#else
	lodsw
	mov	word ptr [bp + di - tbd + o_tbd_tdaddr + 0],ax
	lodsw
	mov	word ptr [bp + di - tbd + o_tbd_tdaddr + 2],ax
#endif
	add	di,#datablk_size
	loop	send2
	pop	di
	pop	si

! Now finally call UNDI to send all packets

	push	bp
	lea	bp,[bp - txbuf]
	mov	ax,#UNDI_TRANSMIT
	call	callpxe
	pop	bp
send9:	pleave
	ret


!
!**************************************************************************
!
! Receive a packet from UNDI
! Input:  1. arg  -  maximum size of local buffer
!         2. arg  -  pointer to local buffer
! Output: number of bytes copied
!
_undi_recv:

	penter	(isr_size + 2)		! setup standard stack frame
	push	es
	push	di

isrbuf	equ	isr_size		! offset to local ISR structure
isrnum	equ	isr_size + 2		! offset to number of bytes read


! Check that we have to process a receive interrupt by calling UNDI ISR.

	mov	ax,#UNDI_ISR_IN_PROCESS
recv1:	call	doisr			! call UNDI
	jc	recv8			! check for error
	cmp	ax,#UNDI_ISR_OUT_DONE
	je	recv8			! check if we are done
	cmp	ax,#UNDI_ISR_OUT_BUSY
	je	recv8			! check for recursive calling
	cmp	ax,#UNDI_ISR_OUT_RECEIVE
	jne	recv2			! we only want receive interrupts

! Check that we received a packet with a valid protocol. If not then read
! the next packet and check again.

	mov	cx,[bp - isrbuf + o_uisr_prottype]
	cmp	cx,#UNDI_PROT_IP
	jb	recv2
	cmp	cx,#UNDI_PROT_RARP
	jbe	recv3
recv2:	mov	ax,#UNDI_ISR_IN_GET_NEXT
	jmp	recv1			! try next interrupt

recv8:	xor	ax,ax			! return with zero bytes
	jmp	recv9

! We finally got a valid packet. First check that the packet will fit into
! the local buffer. Then save the protocol type into the first two bytes
! of the output buffer.

recv3:	mov	bx,[bp - isrbuf + o_uisr_framelength]
	mov	dx,[bp - isrbuf + o_uisr_headerlength]
	sub	bx,dx
	jbe	recv2			! the frame length has to be valid
	add	bx,#2			! add the virtual header size
	getcarg	(ax,0)			! get size of local buffer
	cmp	bx,ax			! check that receive buffer doesnt
	jbe	recv4			! overflow local buffer
	mov	bx,ax
recv4:	mov	[bp - isrnum],bx	! save receive buffer size
	sub	bx,#2

	mov	ax,ds
	mov	es,ax
	getcarg	(di,1)			! get pointer to local buffer
	seg	es
	mov	[di],cx			! save protocol type
	add	di,#2

! We can now cycle through all receive buffers and copy them into the
! output buffer. At this point the registers are used as follows:
!
!	BX     -  remaining frame buffer size
!	DX     -  header size
!	ES:DI  -  pointer to local buffer

recv5:	mov	cx,[bp - isrbuf + o_uisr_buflength]
	sub	cx,dx			! skip media header
	sub	bx,cx			! compute remaining local buffer size
	jae	recv6
	add	cx,bx			! should never exceed local buffer size
	xor	bx,bx

recv6:	cld
	push	ds
	push	si
	lds	si,[bp - isrbuf + o_uisr_frame]
	add	si,dx
	rep				! copy buffer
	movsb
	pop	si
	pop	ds
	or	bx,bx			! check if at end of frame
	jz	recv7

	mov	ax,#UNDI_ISR_IN_GET_NEXT
	call	doisr			! call UNDI to get next buffer
	jc	recv8
	cmp	ax,#UNDI_ISR_OUT_RECEIVE
	jne	recv8			! we only accept receives now
	xor	dx,dx			! header size is now zero
	jmp	recv5			! process next buffer

! We received the full frame now. However, since the local buffer can be
! smaller than the frame we have to skip all following buffers until we
! get UNDI_ISR_OUT_DONE from UNDI. This is important to allow UNDI to
! reenable device interrupts again.

recv7:	mov	ax,#UNDI_ISR_IN_GET_NEXT
	call	doisr			! call UNDI to recycle the last buffer
	jc	recv8
	cmp	ax,#UNDI_ISR_OUT_DONE
	jne	recv7
	mov	ax,[bp - isrnum]	! return number of bytes received
recv9:	pop	di
	pop	es
	pleave
	ret


!
!**************************************************************************
!
! Call UNDI ISR routine.
! Input:  AX  -  ISR function flag
!         BP  -  stack frame of _undi_recv
! Output: AX  -  ISR function flag
!         Carry flag set if error
! Registers changed: AX
!
doisr:

	push	bp
	lea	bp,[bp - isr_size]
	mov	[bp + o_uisr_funcflag],ax	! set ISR function flag
	mov	ax,#UNDI_ISR
	call	callpxe				! call UNDI
	or	ax,ax				! check error code
	stc
	jnz	doisr9
	mov	ax,[bp + o_uisr_funcflag]	! return ISR function flag
	clc
doisr9:	pop	bp
	ret


!
!**************************************************************************
!
! Hardware interrupt routine.
! Input:  none
! Output: none
! Registers changed: none
!
hwint:

! This routine should never get called recursively because hardware
! interrupts should get turned off at the device level by the UNDI
! driver. However, if this doesnt work for some reason, we count the
! interrupt and return. The previous interrupt handler call will then
! recycle this routine at the end. This is necessary because we switch
! stacks for the UNDI call.

	cli
	push	ds
	seg	cs
	mov	ds,[datseg]
	inc	word ptr [intcnt]	! increment interrupt count
	cmp	word ptr [intcnt],#1	! check if this is the first call
	jne	hwint9

	mov	word ptr [oldsp + 0],sp
	mov	word ptr [oldsp + 2],ss
	seg	cs
	mov	ss,[datseg]
	mov	sp,#intstack + STACK_SIZE - 2
	pushall
	push	es

! Call the UNDI driver to handle the interrupt, and check if its ours.

hwint1:	sub	sp,#isr_size
	mov	bp,sp
	mov	word ptr [bp + o_uisr_funcflag],#UNDI_ISR_IN_START
	mov	ax,#UNDI_ISR
	call	callpxe
	or	ax,ax
	jnz	hwint2
	mov	ax,word ptr [bp + o_uisr_funcflag]
hwint2:	add	sp,#isr_size
	or	ax,ax			! check if interrupt is ours
	jnz	hwint5

! Send specific EOI since this interrupt was ours.

	mov	ax,[irqnum]
	cli
	cmp	ah,#$FF			! check if we have to care for
	je	hwint3			! a slave PIC
	xchg	al,ah
	add	al,#CMD_S_EOI
	out	SLAVE_CMD,al		! send specific EOI to slave
	xchg	al,ah
hwint3:	add	al,#CMD_S_EOI
	out	MASTER_CMD,al		! send specific EOI to master
	sti

! Terminate the hardware interrupt processing. If this routine got
! called recursively, recycle it to handle the previous request.

hwint8:	cli
	dec	word ptr [intcnt]
	jnz	hwint1
	pop	es
	popall
#ifdef IS386
	lss	sp,[oldsp]
#else
	mov	ss,word ptr [oldsp + 2]
	mov	sp,word ptr [oldsp + 0]
#endif
hwint9:	pop	ds
	iret


! If the interrupt was not ours we have to pass it down to the old interrupt
! handler because PCI cards can have shared interrupts. This may end invoking
! the BIOS interrupt handler which will turn off the interrupt at the PIC.
! Therefore, we have to reenable interrupts after return from the old handler.

hwint5:	mov	ax,word ptr [oldhwint + 2]	! check if an old handler is
	or	ax,ax				! available
	jz	hwint8

	cli
	pushf
	call	far ptr [oldhwint]	! call old handler

	mov	dx,#MASTER_IMR
	mov	bx,[irqmask]		! get interrupt masks
	or	bl,bl			! check if its on master PIC
	jnz	hwint6
	mov	dx,#SLAVE_IMR		! no, its on the slave
	mov	bl,bh
hwint6:	cli
	in	al,dx			! read interrupt mask register
	test	al,bl			! check if it has been masked
	jz	hwint8
	not	bl
	and	al,bl			! unmask interrupt again
	out	dx,al
	jmp	hwint8


!
!**************************************************************************
!
! Set our hardware interrupt
! Input:  none
! Output: none
! Registers changed: (E)AX, BX, DX, SI
!
setint:

	pushf
	cli

! First save the old interrupt and then set the vector to our own
! interrupt handler routine.

	push	es
	xor	ax,ax
	mov	es,ax
	mov	si,[irqvect]
#ifdef IS386
	mov	ax,cs
	shl	eax,#16
	mov	ax,#hwint
	seg	es
	xchg	[si],eax
	mov	[oldhwint],eax
#else
	mov	ax,#hwint
	seg	es
	xchg	ax,word ptr [si + 0]
	mov	word ptr [oldhwint + 0],ax
	mov	ax,cs
	seg	es
	xchg	ax,word ptr [si + 2]
	mov	word ptr [oldhwint + 2],ax
#endif
	pop	es

! Save the old interrupt mask and unmask our hardware interrupt.

	mov	dx,#MASTER_IMR
	mov	bx,[irqmask]		! get interrupt masks
	or	bl,bl			! check if its on master PIC
	jnz	setin1
	mov	dx,#SLAVE_IMR		! no, its on the slave
	mov	bl,bh
setin1:	in	al,dx			! read interrupt mask register
	mov	[oldmask],al
	not	bl
	and	al,bl			! unmask interrupt
	jmp	setin2
setin2:	jmp	setin3			! wait a little bit
setin3:	out	dx,al
	popf
	ret


!
!**************************************************************************
!
! Reset our hardware interrupt to the original vector
! Input:  none
! Output: BX  -  non-zero if interrupt cant get reset
! Registers changed: (E)AX, BX, DX, SI
!
rstint:

	pushf
	cli

! Set the interrupt mask of our hardware interrupt to its original value.

	mov	dx,#MASTER_IMR
	mov	bx,[irqmask]		! get interrupt masks
	or	bl,bl			! check if its on master PIC
	jnz	rstin1
	mov	dx,#SLAVE_IMR		! no, its on the slave
	mov	bl,bh
rstin1:	in	al,dx			! read interrupt mask register
	and	bl,[oldmask]
	or	al,bl			! mask interrupt
	jmp	rstin2
rstin2:	jmp	rstin3			! wait a little bit
rstin3:	out	dx,al

! Restore the hardware interrupt vector. We cant reset it if the current
! vector is not pointing to our own interrupt handler routine.

	push	es
	xor	ax,ax
	mov	es,ax
	mov	si,[irqvect]
	mov	bx,#1			! set error return value
#ifdef IS386
	mov	ax,cs
	shl	eax,#16
	mov	ax,#hwint		! check if current vector points to
	seg	es			! our int handler
	cmp	eax,[si]
	jne	rstin9
	mov	eax,[oldhwint]
	seg	es			! restore old interrupt
	mov	[si],eax
#else
	seg	es
	mov	ax,word ptr [si + 0]
	seg	es
	mov	dx,word ptr [si + 2]
	cmp	ax,#hwint
	jne	rstin9			! check if current vector points to
	mov	ax,cs			! our int handler
	cmp	dx,ax
	jne	rstin9
	mov	ax,word ptr [oldhwint + 0]
	seg	es
	mov	[si + 0],ax		! restore old interrupt
	mov	ax,word ptr [oldhwint + 2]
	seg	es
	mov	[si + 2],ax
#endif

	xor	bx,bx			! return without error
rstin9:	pop	es
	popf
	ret


!
!**************************************************************************
!
	end

