!**************************************************************************
!*
!* Boot-ROM-Code to load an operating system across a TCP/IP network.
!*
!* Module:  utility.S
!* Purpose: Various functions for the network library, which are
!*          better written in assembler
!* Entries: __htons, __htonl, _ipchksum, _udpchksum
!*
!**************************************************************************
!*
!* Copyright (C) 1995-2003 Gero Kuhlmann <gero@gkminix.han.de>
!*
!*  This program is free software; you can redistribute it and/or modify
!*  it under the terms of the GNU General Public License as published by
!*  the Free Software Foundation; either version 2 of the License, or
!*  any later version.
!*
!*  This program is distributed in the hope that it will be useful,
!*  but WITHOUT ANY WARRANTY; without even the implied warranty of
!*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*  GNU General Public License for more details.
!*
!*  You should have received a copy of the GNU General Public License
!*  along with this program; if not, write to the Free Software
!*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!*
!* $Id: utility.S,v 1.4 2003/01/25 23:29:41 gkminix Exp $
!*


!
!**************************************************************************
!
! Include assembler macros:
!
#include <general.inc>


!
!**************************************************************************
!
! Start code segment.
!
	.text

#ifndef OPT386
	public	__htonl			! define entry points
	public	__htons
#endif
	public	_ipchksum
	public	_udpchksum


!
!**************************************************************************
!
! Convert host byte order into network byte order for short int. This is
! only necessary if the C code is not getting optimized, because otherwise
! this gets optimized away.
! Input:  1. arg  -  host value
! Output: network value
!
#ifndef OPT386
__htons:

	penter	(0)			! setup standard stack frame
	getcarg	(ax,0)			! get host value
	xchg	al,ah			! just swap bytes
	pleave
	ret
#endif


!
!**************************************************************************
!
! Convert host byte order into network byte order for long int.
! Input:  1. arg  -  host value
! Output: network value
!
#ifndef OPT386
__htonl:

	penter	(0)			! setup standard stack frame
	getcarg	(dx,0)			! get low order word into high order reg
	getcarg	(ax,1)			! get high order word into low order reg
	xchg	dl,dh
	xchg	al,ah			! swap bytes in each word
	pleave
	ret
#endif


!
!**************************************************************************
!
! Compute IP header checksum.
! Input:  1. arg  -  pointer to buffer
!         2. arg  -  length of header
! Output: checksum
!
_ipchksum:

	penter	(0)			! setup standard stack frame
	push	si
	getcarg	(si,0)			! get header address
	getcarg	(cx,1)			! get header length
	shr	cx,#1

	cld
	clc
	xor	bx,bx			! add checksum in BX
ipchk1:	lodsw
	adc	bx,ax			! add checksum
	loop	ipchk1
	adc	bx,#0			! add carry
	not	bx
	mov	ax,bx			! prepare return value
	pop	si
	pleave
	ret


!
!**************************************************************************
!
! Compute UDP header checksum.
! Input:  1. arg  -  near pointer to pseudo header
!         2. arg  -  length of pseudo header (has to be even)
!         3. arg  -  near pointer to UDP header
!         4. arg  -  length of UDP header (has to be even)
!         5. arg  -  far pointer to data buffer
!         6. arg  -  length of data buffer
! Output: checksum
!
_udpchksum:

	penter	(0)			! setup standard stack frame
	push	si
	cld

	getcarg	(si,0)			! get pseudo header address
	getcarg	(cx,1)			! get pseudo header length
	shr	cx,#1
	xor	bx,bx			! add checksum in BX
udpck1:	lodsw
	adc	bx,ax			! add checksum
	loop	udpck1
	adc	bx,#0			! add carry

	getcarg	(si,2)			! get UDP header address
	getcarg	(cx,3)			! get UDP header length
	shr	cx,#1
	clc
udpck2:	lodsw
	adc	bx,ax			! add checksum
	loop	udpck2
	adc	bx,#0			! add carry

	push	ds
	getcarg	(si,4)			! get data address
	getcarg	(ds,5)
	getcarg	(cx,6)			! get data length
	mov	dx,cx
	shr	cx,#1
	clc
udpck3:	lodsw
	adc	bx,ax			! add checksum
	loop	udpck3
	adc	bx,#0			! add carry

	test	dx,#$0001		! check if odd number of bytes in data
	jz	udpck4
	lodsb
	xor	ah,ah			! add last byte in
	add	bx,ax
	adc	bx,#0			! add carry
udpck4:	pop	ds

	not	bx
	mov	ax,bx			! prepare return value
	pop	si
	pleave
	ret


!
!**************************************************************************
!
	end

