!**************************************************************************
!*
!* Boot-ROM-Code to load an operating system across a TCP/IP network.
!*
!* Module:  ldram.inc
!* Purpose: Definitions for bootrom RAM area
!* Entries: none
!*
!**************************************************************************
!*
!* Copyright (C) 1998-2003 Gero Kuhlmann <gero@gkminix.han.de>
!*
!*  This program is free software; you can redistribute it and/or modify
!*  it under the terms of the GNU General Public License as published by
!*  the Free Software Foundation; either version 2 of the License, or
!*  any later version.
!*
!*  This program is distributed in the hope that it will be useful,
!*  but WITHOUT ANY WARRANTY; without even the implied warranty of
!*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*  GNU General Public License for more details.
!*
!*  You should have received a copy of the GNU General Public License
!*  along with this program; if not, write to the Free Software
!*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!*
!* $Id: ldram.inc,v 1.5 2003/01/25 23:29:41 gkminix Exp $
!*

#ifndef _LDRAM_INC
#define _LDRAM_INC


!
!**************************************************************************
!
! We need to use some RAM for passing values from the ROM startup code
! to the bootrom code. Since we cant always write into the ROM area itself
! (thats possible with floppy boot or by using DDIM) we have to use some
! other RAM area. The interrupt area from vector $90 to $EF is only used
! by operating systems and the original IBM ROM BASIC, according to Ralf
! Browns interrupt list. We can therefore use that area during boot for
! saving important values. It will be protected by a checksum.
!
RAM_START	equ	($0097 * 4)	! offset to beginning of RAM area
RAM_ID		equ	$0000		! ID of RAM area
RAM_CHKSUM	equ	$0002		! checksum of RAM area
RAM_FLAGS	equ	$0003		! flags
RAM_OLDINT	equ	$0004		! old bootrom interrupt
RAM_EXTADR	equ	$0008		! linear address of new compressed copy
RAM_PNPSTRUCT	equ	$000C		! far ptr to PnP installation struct
RAM_PCIPFA	equ	$0010		! PCI PFA
RAM_PNPCSN	equ	$0012		! PnP CSN

RAM_SIGNATURE	equ	$5A5A		! signature of RAM area
RAM_SIZE	equ	$0014		! size of RAM area (multiple of 4)

RAMFL_BBS	equ	%00000001	! we have been called according to BBS
RAMFL_DDIM	equ	%00000010	! use DDIM
RAMFL_PMM	equ	%00000100	! use PMM


!
!**************************************************************************
!
#endif

