/*
 * string.c  -  String handling which should be in the standard library
 *
 * Copyright (C) 1998-2003 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: string.c,v 1.5 2003/01/25 23:29:44 gkminix Exp $
 */

#include <common.h>
#include <nblib.h>
#include "privlib.h"



/*
 * Find a substring within a string
 */
#ifndef HAVE_STRSTR
char *strstr(haystack, needle)
char *haystack;
char *needle;
{
  size_t nlen = strlen(needle);

  while (strlen(haystack) >= nlen) {
	if (!strncmp(haystack, needle, nlen))
		return(haystack);
	haystack++;
  }
  return(NULL);
}
#endif



/*
 * Handle overlapping memory copies.
 */
#ifndef HAVE_MEMMOVE
voidstar memmove (dest, src, len)
voidstar dest;
voidstar src;
size_t len;
{
  voidstar srcend = (voidstar)((__u8 *)src + len);
  voidstar destend = (voidstar)((__u8 *)dest + len);
  voidstar buf;

  if ((src > dest && src < destend) ||
      (dest > src && dest < srcend)) {
	buf = (voidstar)nbmalloc(len);
	memcpy(buf, src, len);
	memcpy(dest, buf, len);
	free(buf);
  } else if (src != dest) {
	memcpy(dest, src, len);
  }
}
#endif



/*
 * Return the length of the initial segment of a string, which consists
 * entirely of characters in ACCEPT.
 */
#ifndef HAVE_STRSPN
int strspn(str, accept)
char *str;
char *accept;
{
  char *cp;
  int i;

  for (i = 0, cp = str; *cp; i++, cp++)
	if (strchr(accept, *cp) == NULL)
		break;
  return(i);
}
#endif



/*
 * Return the length of the initial segment of a string, which consists
 * entirely of characters not in REJECT.
 */
#ifndef HAVE_STRCSPN
int strcspn(str, reject)
char *str;
char *reject;
{
  char *cp;
  int i;

  for (i = 0, cp = str; *cp; i++, cp++)
	if (strchr(reject, *cp) != NULL)
		break;
  return(i);
}
#endif

